@@ -1,8 +1,8 @@
 use strict;
 use warnings;
 package Dist::Zilla::Plugin::ModuleBuildTiny::Fallback;
-# git description: v0.009-2-g5a5f4be
-$Dist::Zilla::Plugin::ModuleBuildTiny::Fallback::VERSION = '0.010';
+# git description: v0.010-5-ge2bc010
+$Dist::Zilla::Plugin::ModuleBuildTiny::Fallback::VERSION = '0.011';
 # ABSTRACT: Build a Build.PL that uses Module::Build::Tiny, falling back to Module::Build as needed
 # KEYWORDS: plugin installer Module::Build Build.PL toolchain legacy ancient backcompat
 # vim: set ts=8 sw=4 tw=78 et :
@@ -96,7 +96,8 @@ sub gather_files
             if (my $build_pl = first { $_->name eq 'Build.PL' } @{ $self->zilla->files })
             {
                 $self->log_debug('setting aside Build.PL created by ' . blessed($plugin));
-                $files{ blessed $plugin } = $build_pl;
+                $files{ blessed $plugin }{file} = $build_pl;
+                $files{ blessed $plugin }{content} = $build_pl->content;
                 $self->zilla->prune_file($build_pl);
             }
         }
@@ -104,7 +105,7 @@ sub gather_files
 
     # put the Module::Build::Tiny file back in the file list in case other
     # plugins want to add to its content
-    if (my $file = $files{'Dist::Zilla::Plugin::ModuleBuildTiny'}) { push @{ $self->zilla->files }, $file }
+    if (my $file = $files{'Dist::Zilla::Plugin::ModuleBuildTiny'}{file}) { push @{ $self->zilla->files }, $file }
 
     return;
 }
@@ -126,16 +127,22 @@ sub setup_installer
     my ($mb, $mbt) = $self->plugins;
 
     # remove the MBT file that we left in since gather_files
-    if (my $file = $files{'Dist::Zilla::Plugin::ModuleBuildTiny'}) { $self->zilla->prune_file($file) }
+    if (my $file = $files{'Dist::Zilla::Plugin::ModuleBuildTiny'}{file})
+    {
+        $self->zilla->prune_file($file);
+
+        $self->log('something else changed the content of the Module::Build::Tiny version of Build.PL -- maybe you should switch back to [ModuleBuildTiny]?')
+            if $file->content ne $files{'Dist::Zilla::Plugin::ModuleBuildTiny'}{content};
+    }
 
     # let [ModuleBuild] create (or update) the Build.PL file and its content
-    if (my $file = $files{'Dist::Zilla::Plugin::ModuleBuild'}) { push @{ $self->zilla->files }, $file }
+    if (my $file = $files{'Dist::Zilla::Plugin::ModuleBuild'}{file}) { push @{ $self->zilla->files }, $file }
 
     $self->log_debug('generating Build.PL content from [ModuleBuild]');
     $mb->setup_installer;
 
     # find the file object, save its content, and delete it from the file list
-    my $mb_build_pl = $files{'Dist::Zilla::Plugin::ModuleBuild'}
+    my $mb_build_pl = $files{'Dist::Zilla::Plugin::ModuleBuild'}{file}
         || first { $_->name eq 'Build.PL' } @{ $self->zilla->files };
     $self->zilla->prune_file($mb_build_pl);
     my $mb_content = $mb_build_pl->content;
@@ -146,34 +153,57 @@ sub setup_installer
     $mb_content =~ s/^(?!$)/    /mg;
 
     # now let [ModuleBuildTiny] create (or update) the Build.PL file and its content
-    if (my $file = $files{'Dist::Zilla::Plugin::ModuleBuildTiny'}) { push @{ $self->zilla->files }, $file }
+    if (my $file = $files{'Dist::Zilla::Plugin::ModuleBuildTiny'}{file}) { push @{ $self->zilla->files }, $file }
     $self->log_debug('generating Build.PL content from [ModuleBuildTiny]');
     $mbt->setup_installer;
 
     # find the file object, and fold [ModuleBuild]'s content into it
-    my $mbt_build_pl = $files{'Dist::Zilla::Plugin::ModuleBuildTiny'}
+    my $mbt_build_pl = $files{'Dist::Zilla::Plugin::ModuleBuildTiny'}{file}
         || first { $_->name eq 'Build.PL' } @{ $self->zilla->files };
     my $mbt_content = $mbt_build_pl->content;
 
-    # comment out the 'use' line; save the required version
-    $mbt_content =~ s/^(use Module::Build::Tiny ([\d.]+);)$/# $1/m;
-    my $mbt_version = $2;
-    $mbt_content =~ s/This (?:Build.PL|file) /This section /m;
+    # comment out the 'use' line; adjust preamble comments
+    $mbt_content =~ s/^(use Module::Build::Tiny [\d.]+;)$/# $1/m;
+    $mbt_content =~ s/This (?:Build.PL|file) /This section /;
     $mbt_content =~ s/^(?!$)/    /mg;
 
+    # ensure MBT interface is still usable
+    $mbt_content =~ s/(Build_PL)/Module::Build::Tiny::$1/;
+
     my $message = join('', <DATA>);
 
+    my $configure_requires = $self->zilla->prereqs->as_string_hash->{configure}{requires};
+    delete $configure_requires->{perl};
+
+    # prereq specifications don't always provide exact versions - we just weed
+    # those out for now, as this shouldn't occur that frequently.
+    delete @{$configure_requires}{ grep { not version::is_strict($configure_requires->{$_}) } keys %$configure_requires };
+
     $mbt_build_pl->content(
-        <<"FALLBACK",
+        <<"FALLBACK1"
 # This Build.PL for ${\ $self->zilla->name } was generated by
 # ${\ ref $self } ${ \($self->VERSION || '<self>') }
 use strict;
 use warnings;
 
-if (eval 'use Module::Build::Tiny $mbt_version; 1')
-{
-    print "Congratulations, your toolchain understands 'configure_requires'!\\n\\n";
+my %configure_requires = (
+FALLBACK1
+    . join('', map {
+            "    '$_' => '$configure_requires->{$_}',\n"
+        } sort keys %$configure_requires)
+    . <<'FALLBACK2'
+);
 
+my @missing = grep {
+    ! eval "require $_; $_->VERSION($configure_requires{$_}); 1"
+} keys %configure_requires;
+
+if (not @missing)
+{
+    print "Congratulations, your toolchain understands 'configure_requires'!\n\n";
+
+FALLBACK2
+    . <<"FALLBACK3"
 $mbt_content}
 else
 {
@@ -183,7 +213,7 @@ EOW
     sleep 10 if -t STDIN && (-t STDOUT || !(-f STDOUT || -c STDOUT));
 
 $mb_content}
-FALLBACK
+FALLBACK3
     );
 
     return;
@@ -260,6 +290,11 @@ __PACKAGE__->meta->make_immutable;
 #pod normal F<Build.PL> file contents, combining them together into the final
 #pod F<Build.PL> for the distribution.
 #pod
+#pod You are warned if anything else added content into F<Build.PL> (e.g. some
+#pod additional build-time dependency checks), as that code will not run in the
+#pod fallback case. It is up to you to decide whether it is still a good idea to use
+#pod this plugin in this situation.
+#pod
 #pod =for Pod::Coverage before_build gather_files register_prereqs setup_installer
 #pod
 #pod =head1 CONFIGURATION OPTIONS
@@ -309,7 +344,7 @@ Dist::Zilla::Plugin::ModuleBuildTiny::Fallback - Build a Build.PL that uses Modu
 
 =head1 VERSION
 
-version 0.010
+version 0.011
 
 =head1 SYNOPSIS
 
@@ -377,6 +412,11 @@ and L<[ModuleBuild]|Dist::Zilla::Plugin::ModuleBuild> plugins to fetch their
 normal F<Build.PL> file contents, combining them together into the final
 F<Build.PL> for the distribution.
 
+You are warned if anything else added content into F<Build.PL> (e.g. some
+additional build-time dependency checks), as that code will not run in the
+fallback case. It is up to you to decide whether it is still a good idea to use
+this plugin in this situation.
+
 =for Pod::Coverage before_build gather_files register_prereqs setup_installer
 
 =head1 CONFIGURATION OPTIONS
